import { ChangeDetectionStrategy, Component, HostBinding, HostListener, Input } from "@angular/core";
import { SortingDirection } from "../data-operations/sorting-expression.interface";
import { IgxGridAPIService } from "./api.service";
import { IgxColumnComponent } from "./column.component";
var IgxGridHeaderComponent = /** @class */ (function () {
    function IgxGridHeaderComponent(gridAPI) {
        this.gridAPI = gridAPI;
        this.sortDirection = SortingDirection.None;
    }
    Object.defineProperty(IgxGridHeaderComponent.prototype, "styleClasses", {
        get: function () {
            return "igx-grid__th " + this.column.headerClasses;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridHeaderComponent.prototype, "width", {
        get: function () {
            return this.column.width;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridHeaderComponent.prototype, "ascending", {
        get: function () {
            return this.sortDirection === SortingDirection.Asc;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridHeaderComponent.prototype, "descending", {
        get: function () {
            return this.sortDirection === SortingDirection.Desc;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridHeaderComponent.prototype.ngDoCheck = function () {
        this.getSortDirection();
    };
    IgxGridHeaderComponent.prototype.onClick = function (event) {
        event.stopPropagation();
        if (this.column.sortable) {
            this.sortDirection = ++this.sortDirection > SortingDirection.Desc ? SortingDirection.None
                : this.sortDirection;
            this.gridAPI.sort(this.gridID, this.column.field, this.sortDirection);
        }
    };
    IgxGridHeaderComponent.prototype.getSortDirection = function () {
        var _this = this;
        var expr = this.gridAPI.get(this.gridID).sortingExpressions.find(function (x) { return x.fieldName === _this.column.field; });
        if (expr) {
            this.sortDirection = expr.dir;
        }
    };
    IgxGridHeaderComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    preserveWhitespaces: false,
                    selector: "igx-grid-header",
                    template: "\n      <ng-template #defaultColumn>\n          {{ column.header || column.field }}\n      </ng-template>\n\n      <th class=\"igx-grid__th-content\">\n          <span class=\"title\">\n              <ng-container *ngTemplateOutlet=\"column.headerTemplate ? column.headerTemplate : defaultColumn; context: { $implicit: column }\">\n              </ng-container>\n          </span>\n          <igx-grid-filter [column]=\"column\" *ngIf=\"column.filterable\"></igx-grid-filter>\n      </th>\n    "
                },] },
    ];
    /** @nocollapse */
    IgxGridHeaderComponent.ctorParameters = function () { return [
        { type: IgxGridAPIService, },
    ]; };
    IgxGridHeaderComponent.propDecorators = {
        "column": [{ type: Input },],
        "gridID": [{ type: Input },],
        "styleClasses": [{ type: HostBinding, args: ["class",] },],
        "width": [{ type: HostBinding, args: ["style.min-width",] },],
        "ascending": [{ type: HostBinding, args: ["class.asc",] },],
        "descending": [{ type: HostBinding, args: ["class.desc",] },],
        "onClick": [{ type: HostListener, args: ["click", ["$event"],] },],
    };
    return IgxGridHeaderComponent;
}());
export { IgxGridHeaderComponent };
//# sourceMappingURL=grid-header.component.js.map